@@ -34,14 +34,15 @@ from .urls_patterns import URL_ADMIN_SERVER_INFO, URL_ADMIN_CLIENT_AUTHZ_RESOURC
     URL_ADMIN_USER_CLIENT_ROLES_COMPOSITE, URL_ADMIN_USER_GROUP, URL_ADMIN_REALM_ROLES, URL_ADMIN_GROUP_CHILD, \
     URL_ADMIN_USER_CONSENTS, URL_ADMIN_SEND_VERIFY_EMAIL, URL_ADMIN_CLIENT, URL_ADMIN_USER, URL_ADMIN_CLIENT_ROLE, \
     URL_ADMIN_USER_GROUPS, URL_ADMIN_CLIENTS, URL_ADMIN_FLOWS_EXECUTIONS, URL_ADMIN_GROUPS, URL_ADMIN_USER_CLIENT_ROLES, \
-    URL_ADMIN_REALM_IMPORT, URL_ADMIN_USERS_COUNT, URL_ADMIN_FLOWS, URL_ADMIN_GROUP, URL_ADMIN_CLIENT_AUTHZ_SETTINGS, \
+    URL_ADMIN_REALMS, URL_ADMIN_USERS_COUNT, URL_ADMIN_FLOWS, URL_ADMIN_GROUP, URL_ADMIN_CLIENT_AUTHZ_SETTINGS, \
     URL_ADMIN_GROUP_MEMBERS, URL_ADMIN_USER_STORAGE, URL_ADMIN_GROUP_PERMISSIONS, URL_ADMIN_IDPS, \
-    URL_ADMIN_USER_CLIENT_ROLES_AVAILABLE, URL_ADMIN_USERS
+    URL_ADMIN_USER_CLIENT_ROLES_AVAILABLE, URL_ADMIN_USERS, URL_ADMIN_CLIENT_SCOPES, \
+    URL_ADMIN_CLIENT_SCOPES_ADD_MAPPER, URL_ADMIN_CLIENT_SCOPE, URL_ADMIN_CLIENT_SECRETS
 
 
 class KeycloakAdmin:
 
-    def __init__(self, server_url, username, password, realm_name='master', client_id='admin-cli', verify=True):
+    def __init__(self, server_url, username, password, realm_name='master', client_id='admin-cli', verify=True, client_secret_key=None):
         """
 
         :param server_url: Keycloak server url
@@ -50,6 +51,7 @@ class KeycloakAdmin:
         :param realm_name: realm name
         :param client_id: client id
         :param verify: True if want check connection SSL
+        :param client_secret_key: client secret key
         """
         self._username = username
         self._password = password
@@ -58,9 +60,12 @@ class KeycloakAdmin:
 
         # Get token Admin
         keycloak_openid = KeycloakOpenID(server_url=server_url, client_id=client_id, realm_name=realm_name,
-                                         verify=verify)
-        self._token = keycloak_openid.token(username, password)
+                                         verify=verify, client_secret_key=client_secret_key)
 
+        grant_type = ["password"]
+        if client_secret_key:
+            grant_type = ["client_credentials"]
+        self._token = keycloak_openid.token(username, password, grant_type=grant_type)
         self._connection = ConnectionManager(base_url=server_url,
                                              headers={'Authorization': 'Bearer ' + self.token.get('access_token'),
                                                       'Content-Type': 'application/json'},
@@ -127,10 +132,35 @@ class KeycloakAdmin:
         :return: RealmRepresentation
         """
 
-        data_raw = self.connection.raw_post(URL_ADMIN_REALM_IMPORT,
+        data_raw = self.connection.raw_post(URL_ADMIN_REALMS,
                                             data=json.dumps(payload))
         return raise_error_from_response(data_raw, KeycloakGetError, expected_code=201)
 
+    def get_realms(self):
+        """
+        Lists all realms in Keycloak deployment
+
+        :return: realms list
+        """
+        data_raw = self.connection.raw_get(URL_ADMIN_REALMS)
+        return raise_error_from_response(data_raw, KeycloakGetError)
+
+    def create_realm(self, payload, skip_exists=False):
+        """
+        Create a client
+
+        ClientRepresentation: http://www.keycloak.org/docs-api/3.3/rest-api/index.html#_realmrepresentation
+
+        :param skip_exists: Skip if Realm already exist.
+        :param payload: RealmRepresentation
+        :return:  Keycloak server response (UserRepresentation)
+        """
+
+        data_raw = self.connection.raw_post(URL_ADMIN_REALMS,
+                                            data=json.dumps(payload))
+        return raise_error_from_response(data_raw, KeycloakGetError, expected_code=201, skip_exists=skip_exists)
+
+
     def get_users(self, query=None):
         """
         Get users Returns a list of users, filtered according to query parameters
@@ -199,7 +229,7 @@ class KeycloakAdmin:
         :return: user_id
         """
 
-        users = self.get_users(query={"username": username})
+        users = self.get_users(query={"search": username})
         return next((user["id"] for user in users if user["username"] == username), None)
 
     def get_user(self, user_id):
@@ -865,3 +895,57 @@ class KeycloakAdmin:
         data_raw = self.connection.raw_post(URL_ADMIN_USER_STORAGE.format(**params_path),
                                             data=json.dumps(data), **params_query)
         return raise_error_from_response(data_raw, KeycloakGetError)
+
+    def get_client_scopes(self):
+        """
+        Get representation of the client scopes for the realm where we are connected to
+        https://www.keycloak.org/docs-api/4.5/rest-api/index.html#_getclientscopes
+
+        :return: Keycloak server response Array of (ClientScopeRepresentation)
+        """
+
+        params_path = {"realm-name": self.realm_name}
+        data_raw = self.connection.raw_get(URL_ADMIN_CLIENT_SCOPES.format(**params_path))
+        return raise_error_from_response(data_raw, KeycloakGetError)
+
+    def get_client_scope(self, client_scope_id):
+        """
+        Get representation of the client scopes for the realm where we are connected to
+        https://www.keycloak.org/docs-api/4.5/rest-api/index.html#_getclientscopes
+
+        :return: Keycloak server response (ClientScopeRepresentation)
+        """
+
+        params_path = {"realm-name": self.realm_name, "scope-id": client_scope_id}
+        data_raw = self.connection.raw_get(URL_ADMIN_CLIENT_SCOPE.format(**params_path))
+        return raise_error_from_response(data_raw, KeycloakGetError)
+
+
+    def add_mapper_to_client_scope(self, client_scope_id, payload):
+        """
+        Add a mapper to a client scope
+        https://www.keycloak.org/docs-api/4.5/rest-api/index.html#_create_mapper
+
+        :param payload: ProtocolMapperRepresentation
+        :return: Keycloak server Response
+        """
+
+        params_path = {"realm-name": self.realm_name, "scope-id": client_scope_id}
+
+        data_raw = self.connection.raw_post(URL_ADMIN_CLIENT_SCOPES_ADD_MAPPER.format(**params_path), data=json.dumps(payload))
+
+        return raise_error_from_response(data_raw, KeycloakGetError, expected_code=201)
+
+    def get_client_secrets(self, client_id):
+        """
+
+        Get representation of the client secrets
+        https://www.keycloak.org/docs-api/4.5/rest-api/index.html#_getclientsecret
+
+        :param client_id:  id of client (not client-id)
+        :return: Keycloak server response (ClientRepresentation)
+        """
+
+        params_path = {"realm-name": self.realm_name, "id": client_id}
+        data_raw = self.connection.raw_get(URL_ADMIN_CLIENT_SECRETS.format(**params_path))
+        return raise_error_from_response(data_raw, KeycloakGetError)
